@@ -209,7 +209,7 @@ static PetscErrorCode MatMatMultEqual_Private(Mat A, Mat B, Mat C, PetscInt n, P
 }
 
 /*@
-  MatMultEqual - Compares matrix-vector products of two matrices.
+  MatMultEqual - Compares matrix-vector products of two matrices using `n` random vectors
 
   Collective
 
@@ -223,7 +223,12 @@ static PetscErrorCode MatMatMultEqual_Private(Mat A, Mat B, Mat C, PetscInt n, P
 
   Level: intermediate
 
-.seealso: `Mat`, `MatMultAddEqual()`, `MatMultTransposeEqual()`, `MatMultTransposeAddEqual()`, `MatIsLinear()`
+  Note:
+  The tolerance for equality is a generous `PETSC_SQRT_MACHINE_EPSILON` in the norm of the difference of the two computed vectors to
+  allow for differences in the numerical computations. Hence this routine may indicate equality even if there is a small systematic difference
+  between the two matrices.
+
+.seealso: `Mat`, `MatMultAddEqual()`, `MatMultTransposeEqual()`, `MatMultTransposeAddEqual()`, `MatIsLinear()`, `MatEqual()`
 @*/
 PetscErrorCode MatMultEqual(Mat A, Mat B, PetscInt n, PetscBool *flg)
 {
@@ -436,7 +441,6 @@ static PetscErrorCode MatProjMultEqual_Private(Mat A, Mat B, Mat C, PetscInt n,
   PetscReal   norm_abs, norm_rel, tol = PETSC_SQRT_MACHINE_EPSILON;
   PetscInt    i, am, an, bm, bn, cm, cn;
   PetscRandom rdm;
-  PetscScalar none = -1.0;
 
   PetscFunctionBegin;
   PetscCall(MatGetLocalSize(A, &am, &an));
@@ -483,11 +487,11 @@ static PetscErrorCode MatProjMultEqual_Private(Mat A, Mat B, Mat C, PetscInt n,
       PetscCall(MatMultTranspose(B, v1, v3)); /* v3 = Bt*A*B*x */
     }
     PetscCall(VecNorm(v4, NORM_2, &norm_abs));
-    PetscCall(VecAXPY(v4, none, v3));
+    PetscCall(VecAXPY(v4, -1.0, v3));
     PetscCall(VecNorm(v4, NORM_2, &norm_rel));
 
     if (norm_abs > tol) norm_rel /= norm_abs;
-    if (norm_rel > tol) {
+    if (norm_rel > tol || PetscIsInfOrNanReal(norm_rel)) {
       *flg = PETSC_FALSE;
       PetscCall(PetscInfo(A, "Error: %" PetscInt_FMT "-th Mat%sMult() %g\n", i, rart ? "RARt" : "PtAP", (double)norm_rel));
       break;
@@ -613,7 +617,7 @@ PetscErrorCode MatIsLinear(Mat A, PetscInt n, PetscBool *flg)
     PetscCall(VecNorm(s2, NORM_INFINITY, &norm));
     if (norm / normA > 100. * PETSC_MACHINE_EPSILON) {
       *flg = PETSC_FALSE;
-      PetscCall(PetscInfo(A, "Error: %" PetscInt_FMT "-th |A*(ax+y) - (a*A*x+A*y)|/|A(ax+y)| %g > tol %g\n", k, (double)(norm / normA), (double)(100. * PETSC_MACHINE_EPSILON)));
+      PetscCall(PetscInfo(A, "Error: %" PetscInt_FMT "-th |A*(ax+y) - (a*A*x+A*y)|/|A(ax+y)| %g > tol %g\n", k, (double)(norm / normA), (double)(100 * PETSC_MACHINE_EPSILON)));
       break;
     }
   }
