@@ -53,7 +53,7 @@ PetscErrorCode PetscLayoutCreate(MPI_Comm comm, PetscLayout *map)
   PetscCall(PetscNew(map));
   PetscCallMPI(MPI_Comm_size(comm, &(*map)->size));
   (*map)->comm        = comm;
-  (*map)->bs          = -1;
+  (*map)->bs          = 1;
   (*map)->n           = -1;
   (*map)->N           = -1;
   (*map)->range       = NULL;
@@ -84,7 +84,9 @@ PetscErrorCode PetscLayoutCreate(MPI_Comm comm, PetscLayout *map)
   Level: advanced
 
   Note:
-$ PetscLayoutCreateFromSizes(comm, n, N, bs, &layout);
+.vb
+  PetscLayoutCreateFromSizes(comm, n, N, bs, &layout);
+.ve
   is a shorthand for
 .vb
   PetscLayoutCreate(comm, &layout);
@@ -170,11 +172,12 @@ PetscErrorCode PetscLayoutCreateFromRanges(MPI_Comm comm, const PetscInt range[]
     PetscCall(PetscArraycpy(map->range, range, map->size + 1));
     break;
   case PETSC_USE_POINTER:
-    map->range_alloc = PETSC_FALSE;
-    break;
-  default:
+    map->range_alloc = PETSC_FALSE; /* fall through */
+  case PETSC_OWN_POINTER:
     map->range = (PetscInt *)range;
     break;
+  default:
+    SETERRQ(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Received invalid PetscCopyMode somehow");
   }
   map->rstart = map->range[rank];
   map->rend   = map->range[rank + 1];
@@ -184,8 +187,8 @@ PetscErrorCode PetscLayoutCreateFromRanges(MPI_Comm comm, const PetscInt range[]
     PetscInt tmp;
     PetscCallMPI(MPIU_Allreduce(&map->n, &tmp, 1, MPIU_INT, MPI_SUM, map->comm));
     PetscCheck(tmp == map->N, PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Sum of local lengths %" PetscInt_FMT " does not equal global length %" PetscInt_FMT ", my local length %" PetscInt_FMT ". The provided PetscLayout is wrong.", tmp, map->N, map->n);
-    if (map->bs > 1) PetscCheck(map->n % map->bs == 0, PETSC_COMM_SELF, PETSC_ERR_PLIB, "Local size %" PetscInt_FMT " must be divisible by blocksize %" PetscInt_FMT, map->n, map->bs);
-    if (map->bs > 1) PetscCheck(map->N % map->bs == 0, map->comm, PETSC_ERR_PLIB, "Global size %" PetscInt_FMT " must be divisible by blocksize %" PetscInt_FMT, map->N, map->bs);
+    PetscCheck(map->n % map->bs == 0, PETSC_COMM_SELF, PETSC_ERR_PLIB, "Local size %" PetscInt_FMT " must be divisible by blocksize %" PetscInt_FMT, map->n, map->bs);
+    PetscCheck(map->N % map->bs == 0, map->comm, PETSC_ERR_PLIB, "Global size %" PetscInt_FMT " must be divisible by blocksize %" PetscInt_FMT, map->N, map->bs);
   }
   /* lock the layout */
   map->setupcalled = PETSC_TRUE;
@@ -235,15 +238,15 @@ PetscErrorCode PetscLayoutSetUp(PetscLayout map)
              map->oldn, map->oldN, map->n, map->N);
   if (map->setupcalled) PetscFunctionReturn(PETSC_SUCCESS);
 
-  if (map->n > 0 && map->bs > 1) PetscCheck(map->n % map->bs == 0, PETSC_COMM_SELF, PETSC_ERR_PLIB, "Local size %" PetscInt_FMT " must be divisible by blocksize %" PetscInt_FMT, map->n, map->bs);
-  if (map->N > 0 && map->bs > 1) PetscCheck(map->N % map->bs == 0, map->comm, PETSC_ERR_PLIB, "Global size %" PetscInt_FMT " must be divisible by blocksize %" PetscInt_FMT, map->N, map->bs);
+  PetscCheck(map->n < 0 || map->n % map->bs == 0, PETSC_COMM_SELF, PETSC_ERR_PLIB, "Local size %" PetscInt_FMT " must be divisible by blocksize %" PetscInt_FMT, map->n, map->bs);
+  PetscCheck(map->N < 0 || map->N % map->bs == 0, map->comm, PETSC_ERR_PLIB, "Global size %" PetscInt_FMT " must be divisible by blocksize %" PetscInt_FMT, map->N, map->bs);
 
   PetscCallMPI(MPI_Comm_rank(map->comm, &rank));
-  if (map->n > 0) map->n = map->n / PetscAbs(map->bs);
-  if (map->N > 0) map->N = map->N / PetscAbs(map->bs);
+  if (map->n > 0) map->n = map->n / map->bs;
+  if (map->N > 0) map->N = map->N / map->bs;
   PetscCall(PetscSplitOwnership(map->comm, &map->n, &map->N));
-  map->n = map->n * PetscAbs(map->bs);
-  map->N = map->N * PetscAbs(map->bs);
+  map->n = map->n * map->bs;
+  map->N = map->N * map->bs;
   if (!map->range) PetscCall(PetscMalloc1(map->size + 1, &map->range));
   PetscCallMPI(MPI_Allgather(&map->n, 1, MPIU_INT, map->range + 1, 1, MPIU_INT, map->comm));
 
@@ -349,7 +352,7 @@ PetscErrorCode PetscLayoutSetISLocalToGlobalMapping(PetscLayout in, ISLocalToGlo
     PetscInt bs;
 
     PetscCall(ISLocalToGlobalMappingGetBlockSize(ltog, &bs));
-    PetscCheck(in->bs <= 0 || bs == 1 || in->bs == bs, in->comm, PETSC_ERR_PLIB, "Blocksize of layout %" PetscInt_FMT " must match that of mapping %" PetscInt_FMT " (or the latter must be 1)", in->bs, bs);
+    PetscCheck(in->bs == 1 || bs == 1 || in->bs == bs, in->comm, PETSC_ERR_PLIB, "Blocksize of layout %" PetscInt_FMT " must match that of mapping %" PetscInt_FMT " (or the latter must be 1)", in->bs, bs);
   }
   PetscCall(PetscObjectReference((PetscObject)ltog));
   PetscCall(ISLocalToGlobalMappingDestroy(&in->mapping));
@@ -364,7 +367,7 @@ PetscErrorCode PetscLayoutSetISLocalToGlobalMapping(PetscLayout in, ISLocalToGlo
 
   Input Parameters:
 + map - pointer to the map
-- n   - the local size
+- n   - the local size, pass `PETSC_DECIDE` (the default) to have this value determined by the global size set with `PetscLayoutSetSize()`
 
   Level: developer
 
@@ -374,7 +377,7 @@ PetscErrorCode PetscLayoutSetISLocalToGlobalMapping(PetscLayout in, ISLocalToGlo
 PetscErrorCode PetscLayoutSetLocalSize(PetscLayout map, PetscInt n)
 {
   PetscFunctionBegin;
-  PetscCheck(map->bs <= 1 || (n % map->bs) == 0, map->comm, PETSC_ERR_ARG_INCOMP, "Local size %" PetscInt_FMT " not compatible with block size %" PetscInt_FMT, n, map->bs);
+  PetscCheck(n % map->bs == 0, map->comm, PETSC_ERR_ARG_INCOMP, "Local size %" PetscInt_FMT " not compatible with block size %" PetscInt_FMT, n, map->bs);
   map->n = n;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -412,7 +415,7 @@ PetscErrorCode PetscLayoutGetLocalSize(PetscLayout map, PetscInt *n)
 
   Input Parameters:
 + map - pointer to the map
-- n   - the global size
+- n   - the global size, use `PETSC_DETERMINE` (the default) to have this value computed as the sum of the local sizes set with `PetscLayoutSetLocalSize()`
 
   Level: developer
 
@@ -469,7 +472,7 @@ PetscErrorCode PetscLayoutGetSize(PetscLayout map, PetscInt *n)
 PetscErrorCode PetscLayoutSetBlockSize(PetscLayout map, PetscInt bs)
 {
   PetscFunctionBegin;
-  if (bs < 0) PetscFunctionReturn(PETSC_SUCCESS);
+  PetscCheck(bs > 0, PETSC_COMM_SELF, PETSC_ERR_ARG_INCOMP, "Block size %" PetscInt_FMT " must be positive", bs);
   PetscCheck(map->n <= 0 || (map->n % bs) == 0, PETSC_COMM_SELF, PETSC_ERR_ARG_INCOMP, "Local size %" PetscInt_FMT " not compatible with block size %" PetscInt_FMT, map->n, bs);
   if (map->mapping) {
     PetscInt obs;
@@ -503,7 +506,7 @@ PetscErrorCode PetscLayoutSetBlockSize(PetscLayout map, PetscInt bs)
 PetscErrorCode PetscLayoutGetBlockSize(PetscLayout map, PetscInt *bs)
 {
   PetscFunctionBegin;
-  *bs = PetscAbs(map->bs);
+  *bs = map->bs;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
